
// lexScanner.h is a header file exported for the use of scanner module
//The compiler checks whether SCNNER_H is defined already to see
//whether this header file is already included.
//If it is, skip everything all the way to the matching #endif in the end.
#ifndef LEXICAL_ANALYSIS_H   

//If LEXICAL_ANALYSIS_H  not defined yet, define it here to avoid inclusion
//of lexScanner.h again later during the compilation of the module.
#define LEXICAL_ANALYSIS_H 
#include <vector>
#include <string>
using namespace std;

//****************************************************************************
// The following defines an enumeration type to 
// symbolically encode the categories of tokens (as integers implicitly).
//****************************************************************************


//*******************************************
//The Lexical scanner class
//*******************************************
class LexScanner
{public:
	typedef enum 
	{
		KEYWORD,
		// keywords of BIOLA, like return, display, read, if, loopBegin, loopEnd, function, functionCall, 

		ID_NAME,	
		// identifiers, i.e. names of variables, labels, and functions

		ASSIGNMENT_OP,
		// =, the assignment operator		

		NUMERICAL_OP,
		// numerical operators	 (such as +, -, *, /)	

		LOGICAL_OP,
		// numerical operators	 (such as !, ||, &&)	

		RELATIONAL_OP,
		// relational operators	 (such as >, ==, <, >=, <=)
		
		NUMERICAL_LITERAL,	
		// numerical values (such as 2.45)	
		
		STRING_LITERAL,	
		// string literals (like "Hi Hi", in a pair of double quotes)

		COLON,
		// : to signal the end of function name and parameters declaration 

		SEMICOLON,			// ; to signal the end of statement 

		COMMA,				// a comma ,

		PERIOD,				// a period ,

		LEFT_PARENTHESIS,	// a left parenthesis (

		RIGHT_PARENTHESIS,	// a left parenthesis )

		LEFT_CURLYBRACE,	// a left curly brace {

		RIGHT_CURLYBRACE,	// a right curly brace }

		COMMENT	,	// The token // that starts a line of comments

		UNKNOWN,		// Something that we cannot recognize its category

		//Ignore the following three categories, not needed for at the moment.
		//They are simply the refinement of the ID_NAME category
		FUN_ID_NAME,
		// ignore this category, not needed at the moment
		// identifiers, specifically as functions, such as myFun1

		VAR_ID_NAME,
		// ignore this category, not needed at the moment
		// identifiers, specifically as names of variables, such as x1

		LABEL_ID_NAME
		// ignore this category, not needed at the moment
		// identifiers, specifically as names of labels, such as loop1

	} TokenCategory;


	//****************************************************************************
	// define a mnemonic name for the type to store tokens of one line of statement
	//****************************************************************************
	typedef vector<string> PerLineTokenVector;

	//****************************************************************************
	// define a mnemonic name for the type to store tokens of lines of statement
	//****************************************************************************
	typedef vector<PerLineTokenVector> VectOfTokenVects;

	//****************************************************************************
	// define a mnemonic name for the type to store categories of tokens 
	// of one line of statement
	//****************************************************************************
	typedef vector<TokenCategory> PerLineTokenCategoryVector;

	//****************************************************************************
	// define a mnemonic name for the type to store categories of tokens 
	// of one line of statement
	//****************************************************************************
	typedef vector<PerLineTokenCategoryVector> VectOfCategoryVects;



	//****************************************************************************
	// Prototypes of functions to test the category of a given token
	//****************************************************************************
		bool isKEYWORD(const string & token);
		bool isASSIGNMENT_OP(const string & token);
		bool isCOLON(const string & token);
		bool isCOMMA(const string & token);
		bool isCOMMENT(const string & token);
		bool isID_NAME(const string & token);
		bool isLEFT_PARENTHESIS(const string & token);
		bool isRIGHT_PARENTHESIS(const string & token);
		bool isLEFT_CURLYBRACE(const string & token);
		bool isPERIOD(const string & token);
		bool isRIGHT_CURLYBRACE(const string & token);
		bool isNUMERICAL_LITERAL(const string & token);
		bool isNUMERICAL_OP(const string & token);
		bool isRELATIONAL_OP(const string & token);
		bool isSEMICOLON(const string & token);
		bool isSTRING_LITERAL(const string & token);
		bool isLOGICAL_OP(const string & token);

	//****************************************************************************
	// Prototypes of the getLexicalInfoAllLines functions 
	// to get two kinds of lexical information from 
	// all lines of a program, i.e. 
	// (1) the vector of token vectors of all lines and
	// (2) the vector of category vectors of all lines.
	//
	// Implementation:
	// It should be implementated by appropriately calling the functions: 
	//		getPerLineTokenVectFromOneStringObject, and
	//		getLexCategoryVectorFromTokenVector
	//****************************************************************************
		void getLexicalInfoAllLines(
						const vector<string> & linesOfStatements, 
						VectOfTokenVects & tokenVectors, 
						VectOfCategoryVects & categoryVectors);


	//**********************************************************************
	// getPerLineTokenVectFromOneCharArray and 
	// getPerLineTokenVectFromOneStringObject
	// recognize all the tokens in a single line of statement and
	// then store tokens as a vector of strings.
	//
	// getLexCategoryVectorFromTokenVector
	// recognizes the categories of the tokens given
	// a vector of the tokens in a single line of statement.
	//****************************************************************************
		void getPerLineTokenVectFromOneCharArray
								(
									const char* statement, 
									PerLineTokenVector & tokenVector
								);

		void getPerLineTokenVectFromOneStringObject
								(	const string statement, 
									PerLineTokenVector & tokenVector
								);

		void getLexCategoryVectorFromTokenVector(
			const PerLineTokenVector & tokenVector,
			PerLineTokenCategoryVector & categoryVector		);


	//****************************************************************************
	// Prototype of the function to display lexical information strored in
	// a vector of token vectors of all lines of statements, and 
	// a vector of category vectors of all lines of statements.
	//****************************************************************************
		void displayLexicalInfo(
						VectOfTokenVects & tokenVectos, 
						VectOfCategoryVects & categoryVectors);

};

#endif // Matching the starting #ifndef LEXICAL_ANALYSIS_H  